/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.DisabledIf;

import de.ozgcloud.common.test.DbInitializer;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangService;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;

@DisabledIf(expression = "${elasticTests.disabled:false}", reason = "Wegen hohem Speicherverbrauch des testcontainers")
class SearchIndexInitializerITCase {
	private static final int FOR_500_MS = 500;

	@Nested
	@ITCase
	class TestNotConfigured {

		@MockBean
		private VorgangService vorgangService;

		@Test
		void shouldNotStartIndexer() throws IOException {
			verify(vorgangService, after(FOR_500_MS).never()).findAll();
		}
	}

	@Nested
	@SearchITCase
	class TestConfiguredNotEnabled {

		@MockBean
		private SearchService elasticService;

		@MockBean
		private VorgangService vorgangService;

		@Test
		void shouldNotStartIndexer() throws IOException {
			verify(vorgangService, after(FOR_500_MS).never()).findAll();
			verify(elasticService, after(FOR_500_MS).never()).updateVorgang(any());
		}
	}

	@Disabled("check test")
	@Nested
	@ITCase
	@ContextConfiguration(initializers = { DbInitializer.class, SearchInitializer.class })
	@ActiveProfiles({ "itcase", "with_db", "initSearchIndex" })
	class TestConfiguredEnabled {
		@SpyBean
		private SearchService elasticService;

		@SpyBean
		private SearchIndexInitializer searchInitializer;

		@Autowired
		private MongoTemplate mongoTemplate;

		@BeforeEach
		void init() {
			mongoTemplate.dropCollection(Vorgang.COLLECTION_NAME);

			doNothing().when(searchInitializer).stopApplication();
		}

		@Test
		void shouldNotStartIndexerWhenNoVorgang() throws IOException {
			searchInitializer.onApplicationEvent(null);

			verify(elasticService, after(FOR_500_MS).never()).addVorgang(any());
		}

		@Test
		void shouldStartIndexer() throws IOException {
			mongoTemplate.save(VorgangTestFactory.create(), Vorgang.COLLECTION_NAME);

			searchInitializer.onApplicationEvent(null);

			verify(elasticService, after(FOR_500_MS).atLeastOnce()).addVorgang(any());
		}
	}
}
