package de.ozgcloud.vorgang.vorgang;

import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;

class VorgangStubMapperTest {

	@InjectMocks
	private VorgangStubMapper mapper = Mappers.getMapper(VorgangStubMapper.class);

	@Nested
	class TestFormVorgang {
		@Test
		void shouldMapFields() {
			var stub = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(stub).usingRecursiveComparison()
					.ignoringFields("deletedAt", "status")
					.isEqualTo(VorgangStubTestFactory.create());
		}

		@Test
		void shouldHaveStatusDELETED() {
			var stub = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(stub.getStatus()).isEqualTo(Vorgang.Status.DELETED);
		}

		@Test
		void shouldHaveDeletedAtOnNow() {
			var stub = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(stub.getDeletedAt()).isCloseTo(ZonedDateTime.now(), byLessThan(500, ChronoUnit.MILLIS));
		}
	}

}
